:root {
  --md-primary-fg-color: #3f51b5;
  --md-accent-fg-color: #ff4081;
}

.md-typeset pre > code {
  border-radius: 4px;
}

.md-typeset .admonition {
  border-radius: 4px;
}

.md-nav__item--active > .md-nav__link {
  font-weight: 600;
}

.md-typeset table:not([class]) {
  display: table;
  border-collapse: collapse;
}

.md-typeset table:not([class]) th {
  background-color: var(--md-default-fg-color--lightest);
  font-weight: 700;
}

.md-typeset pre > code {
  white-space: pre;
  font-size: 0.80rem;
  line-height: 1.45;
}

.md-typeset code:not(pre code) {
  white-space: break-spaces;
  font-size: 0.85em;
  padding: 0.15em 0.35em;
}

.md-typeset pre {
  padding: 0.6rem 0.9rem;
  tab-size: 2;
}

.md-typeset h1 + p,
.md-typeset h2 + p {
  margin-top: .2rem;
}

.md-typeset .mermaid {
  overflow: auto;
}

